/*
 Basic ESP8266 MQTT example
 This sketch demonstrates the capabilities of the pubsub library in combination
 with the ESP8266 board/library.
 It connects to an MQTT server then:
  - publishes "hello world" to the topic "outTopic" every two seconds
  - subscribes to the topic "inTopic", printing out any messages
    it receives. NB - it assumes the received payloads are strings not binary
  - If the first character of the topic "inTopic" is an 1, switch ON the ESP Led,
    else switch it off
 It will reconnect to the server if the connection is lost using a blocking
 reconnect function. See the 'mqtt_reconnect_nonblocking' example for how to
 achieve the same result without blocking the main loop.
 To install the ESP8266 board, (using Arduino 1.6.4+):
  - Add the following 3rd party board manager under "File -> Preferences -> Additional Boards Manager URLs":
       http://arduino.esp8266.com/stable/package_esp8266com_index.json
  - Open the "Tools -> Board -> Board Manager" and click install for the ESP8266"
  - Select your ESP8266 in "Tools -> Board"
*/
//#include <ESP8266WiFi.h>
#include <WiFi.h>
#include <PubSubClient.h>
#include <ArduinoJson.h>
#include <TimeLib.h>// 具体用途
//#include "aliyun_mqtt.h"
DynamicJsonDocument doc(1024);

// Update these with values suitable for your network.

const char* ssid = "Bobphone";
const char* password = "hellofab";
const char* mqtt_server = "C6PZ5706SB.iotcloud.tencentdevices.com";
//更改

WiFiClient espClient;
PubSubClient client(espClient);
unsigned long lastMsg = 0;
#define MSG_BUFFER_SIZE (50)
#define pmdata A0
char msg[MSG_BUFFER_SIZE];
int val = 0;
int value=0;
int LED=10;
bool ledState = false; // 全局变量来追踪LED状态

void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
    //Serial.println("WiFi does not connect, try again ...");
  }

  randomSeed(micros());

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

}

void callback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
 
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);
  }
  Serial.println();
  DeserializationError error = deserializeJson(doc, payload);
    // Test if parsing succeeds.
  if (error) {
    Serial.print("deserializeJson() failed: ");
    Serial.println(error.f_str());
    return;
  }
  else{
    //控制LED亮灭
    if (doc["clientToken"]!="00000001") {
      if (doc["params"]["_mesh_generic_onoff"] == 1) {
        Serial.print("Power On ");
        digitalWrite(LED,HIGH);
        ledState = true;
        delay(500);
      } 
      else {
        Serial.print("Power Off ");
        digitalWrite(LED,LOW);
        ledState = false;
        delay(500);
      }
    }
  }

}
// void publish() {

//   // Convert the voltage into the temperature in degree Celsius:
  

//   doc["method"] = "report";
//   doc["clientToken"] = "123";
//   doc["timestamp"] = now();//需要配套timelib.h
//   doc["params"]["pmdata"] = String(pmdata, 2);
//   char output[1024];
//   serializeJson(doc, output);
//   client.publish("$thing/up/property/C6PZ5706SB/001switch", output);
//  //client.subscribe("$thing/down/property/C6PZ5706SB/001switch");//更改

//   // Print pmdata in the Serial Monitor:
//   Serial.println(output);
//   Serial.println("empty");

//   delay(1000); // wait a second between readings
// }


void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Create a random client ID
    // Attempt to connect
    if (client.connect("C6PZ5706SB001switch|securemode=3,signmethod=hmacsha1|", "C6PZ5706SB001switch;12010126;MO2Z3;1701174252", "af2365e9b8ce4df82fba569c58bb189e6ca532f5d8b55cb95df9a4bc44258644;hmacsha256")) {
     //更改
      Serial.println("connected");
      // Once connected, publish an announcement...
      // client.publish("$thing/up/property/RA3ZZK17YI/esp32", "123");
   //   client.publish("thing/up/property/C6PZ5706SB/001switch", potString);
      // ... and resubscribe
      client.subscribe("$thing/down/property/C6PZ5706SB/001switch");//更改
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}
void setup() {
 
  Serial.begin(115200);
  //LED示意项目启动
  digitalWrite(LED,HIGH);
  delay(500);
  digitalWrite(LED,LOW);
  delay(500);
  digitalWrite(LED,HIGH);
  delay(1000);
  digitalWrite(LED,LOW);
  setup_wifi();
  //LED示意Wi-Fi连接成功
  digitalWrite(LED,HIGH);
  delay(3000);
  digitalWrite(LED,LOW);
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
  pinMode(LED,OUTPUT);
  pinMode(pmdata,INPUT);
}

void loop() {
  

  if (!client.connected()) {
    reconnect();
  }
  client.loop();
  //publish();
  // int val = analogRead(pmdata);
  // char potString[8];
  // itoa(val, potString, 10);
  // client.publish("$thing/up/property/C6PZ5706SB/001switch", potString);
  //client.subscribe("$thing/down/property/C6PZ5706SB/001switch");

  


  unsigned long now = millis();
  if (now - lastMsg > 2000) {
    lastMsg = now;
    digitalWrite(LED, ledState ? HIGH : LOW);
    //publish();  //https://www.cnblogs.com/imgss/p/16280737.html
    //mqtt_interval_post();//https://www.nexmaker.com/doc/10IOT/NodeMCUESP8266_ALiYun.html
    ++value;
    int val = analogRead(pmdata); // 读取电位器的值
    Serial.print("Read value: "); // 调试打印
    Serial.println(val);

    // 构造JSON消息
    DynamicJsonDocument doc(1024);
    doc["method"] = "report";
    doc["clientToken"] = "123";
    doc["timestamp"] = 1628646783;
    //doc["clientToken"] = "v2530232175YpImx::c36afa70-bd6c-4e50-a535-38918086563f";
    doc["params"]["_mesh_light_lightness"] = val;
    //doc["params"]["_mesh_generic_onoff"] = 1; // 根据实际情况调整
    //doc["params"]["_mesh_generic_onoff"] = ledState ? 1 : 0; // 根据LED状态设置

    char jsonMessage[512];
    serializeJson(doc, jsonMessage);
    Serial.print("Publishing message: ");
    Serial.println(jsonMessage);
    client.publish("$thing/up/property/C6PZ5706SB/001switch", jsonMessage);
    //client.publish("$thing/up/property/C6PZ5706SB/001switch", jsonMessage);

    // snprintf(msg, MSG_BUFFER_SIZE, "hello world #%ld", value);
    // Serial.print("Publish message: ");
    // Serial.println(msg);
    // client.publish("$thing/up/property/RA3ZZK17YI/esp32", msg);
  }
   
}